#!/usr/bin/env bash

# Change this according to your set up
INSTALL_DBT2=${INSTALL_DBT2:-"NO"}
INSTALL_SYSBENCH=${INSTALL_SYSBENCH:-"NO"}
INSTALL_TPCE=${INSTALL_TPCE:-"NO"}
INSTALL_TPCC=${INSTALL_TPCC:-"NO"}
INSTALL_TPCW=${INSTALL_TPCW:-"NO"}
#


TOP_DIR=$PWD
if test -f $TOP_DIR/config.sh
then
	source $TOP_DIR/config.sh
else
	source $TOP_DIR/config.sh.example
fi

export PATH=$PATH:$TOP_DIR/db/${mysql_server}/bin

test -z "$force_mysql_server" || mysql_server="$force_mysql_server"

# Install MySQL
function install_mysql()
{
mysql_sources=$TOP_DIR/src/${mysql_server}

if test -d $mysql_sources
then
	echo "Compiling MySQL"
	mkdir -p "$TOP_DIR/db/${mysql_server}"
	cd $TOP_DIR/src
	if test -d ${mysql_server}
	then
		if test -f configure_${mysql_server}.sh
		then
			. configure_${mysql_server}.sh && make all install
		else
			echo "Can't find configure script configure_${mysql_server}.sh"
			exit -1
		fi
	fi
else
	echo "Can't find MySQL ${mysql_server}"
	echo "Either put MySQL sources to $mysql_sources or install binaries to $TOP_DIR/db/${mysql_server}"
	exit -1
fi
}

# Install sysbench
function install_sysbench()
{
if test -d $TOP_DIR/src/sysbench
then
	echo "Compiling sysbench"
	mkdir -p "$TOP_DIR/benchmarks/sysbench"
	cd $TOP_DIR/src/sysbench
	./configure --prefix=$TOP_DIR/benchmarks/sysbench --with-mysql=$TOP_DIR/db/${mysql_server}
	if [ $? -eq 0 ]
	then
		make all install
	else
		echo "Error while configuring sysbench"
		exit -1
	fi
else
	echo "Can't find sysbench sources in $TOP_DIR/src/sysbench"
	exit -1
fi
}


# Install DBT2
function install_dbt2()
{
#dbt2_version=dbt2-0.37
dbt2_version=dbt2

if test -d $TOP_DIR/src/${dbt2_version}
then
	echo "Compiling DBT2"
	mkdir -p "$TOP_DIR/benchmarks/dbt2"
	if test -d $TOP_DIR/src/$dbt2_version
	then
		cd $TOP_DIR/src/${dbt2_version}
		./configure --prefix=$TOP_DIR/benchmarks/dbt2 --with-mysql --with-mysql-includes=$TOP_DIR/db/${mysql_server}/include/mysql --with-mysql-libs=$TOP_DIR/db/${mysql_server}/lib/mysql && make all install
	else
		echo "Put sources of $dbt2_version to the directory $TOP_DIR/src"
		exit -1
	fi
else
	echo "Can't find DBT2 sources in $TOP_DIR/src/${dbt2_version}"
	exit -1
fi
}

# Install TPC-E
function install_tpce()
{
tpce_version="EGen_v1.5.1"
if test -d $TOP_DIR/src/${tpce_version}
then
	echo "Compiling TPC-E"
	mkdir -p "$TOP_DIR/benchmarks/tpce/bin" "$TOP_DIR/benchmarks/tpce/scripts" "$tpce_flat_in"
	cd $TOP_DIR/src/${tpce_version}
	cd prj
	cp Makefile.new Makefile
	make
	cd ..
	install -v bin/EGenLoader $TOP_DIR/benchmarks/tpce/bin
	install -v bin/EGenSimpleTest $TOP_DIR/benchmarks/tpce/bin
	cd scripts/mysql
	cd $TOP_DIR/src/${tpce_version}/flat_in/
	for i in `ls`
	do
		install -v -m 644 $i $tpce_flat_in
	done
	cd $TOP_DIR
else
	echo "Can't find TPC-E sources in $TOP_DIR/src/${tpce_version}"
        exit -1
fi

}

# Install TPC-C
function install_tpcc()
{
tpcc_version="tpcc-mysql"
if test -d $TOP_DIR/src/${tpcc_version}
then
	echo "Compiling TPC-C"
	mkdir -p "$TOP_DIR/benchmarks/tpcc/bin" "$TOP_DIR/benchmarks/tpcc/scripts"
	cd $TOP_DIR/src/${tpcc_version}
	cd src
	make
	cd ..
	install -v tpcc_load $TOP_DIR/benchmarks/tpcc/bin
	install -v tpcc_start $TOP_DIR/benchmarks/tpcc/bin
	for i in *.sql
	do
		install -v -m 644 $i $TOP_DIR/benchmarks/tpcc/scripts
	done
	cd $TOP_DIR
else
	echo "Can't find TPC-C sources in $TOP_DIR/src/${tpcc_version}"
        exit -1
fi

}

# Install TPC-W
function install_tpcw()
{
tpcw_version="dbt1-v2.1"
if test -d $TOP_DIR/src/${tpcw_version}
then
	echo "Compiling TPC-W"
	cd $TOP_DIR/src/${tpcw_version}
	./configure --prefix=$TOP_DIR/benchmarks/tpcw --with-mysql && make all install
	
	install -v datagen/datagen $TOP_DIR/benchmarks/tpcw/bin
	install -v dbdriver/dbdriver $TOP_DIR/benchmarks/tpcw/bin
	install -v cache/appCache $TOP_DIR/benchmarks/tpcw/bin
	install -v appServer/appServer $TOP_DIR/benchmarks/tpcw/bin
	install -v -m 755 scripts/mysql/build_db.sh $TOP_DIR/benchmarks/tpcw/scripts
	install -v -m 644 datagen/country.data.sapdb $TOP_DIR/benchmarks/tpcw/scripts
	cd $TOP_DIR
else
	echo "Can't find TPC-W sources in $TOP_DIR/src/${tpcw_version}"
        exit -1
fi

}
# installs config.sh and chart.conf.sh
function install_configs()
{
oldpwd=`pwd`
cd $TOP_DIR
for config in config.sh chart.conf.sh
do
	if ! test -h $config
	then
		echo "Installing $config"
		if test -f $config.example; then ln -s $config.example $config; fi
	fi
done

chmod +x chart.sh
cd $oldpwd

}


if ! test -f $TOP_DIR/db/${mysql_server}/bin/mysql
then
	install_mysql
else
	echo "MySQL ${mysql_server} is installed"
fi

if [ $INSTALL_DBT2 = "YES" ]
then
	if ! test -f $TOP_DIR/benchmarks/dbt2/bin/driver
	then
		install_dbt2
	else
		echo "DBT2 is installed"
	fi
fi

if [ $INSTALL_SYSBENCH = "YES" ]
then
	if ! test -f $TOP_DIR/benchmarks/sysbench/bin/sysbench
	then
		install_sysbench
	else
		echo "Sysbench is installed"
	fi
fi

if [ $INSTALL_TPCE = "YES" ]
then
	if ! test -f $TOP_DIR/benchmarks/tpce/bin/EGenLoader
	then
		install_tpce
	else
		echo "TPC-E is installed"
	fi
fi

if [ $INSTALL_TPCC = "YES" ]
then
	if ! test -f $TOP_DIR/benchmarks/tpcc/bin/tpcc_load
	then
		install_tpcc
	else
		echo "TPC-C is installed"
	fi
fi

if [ $INSTALL_TPCW = "YES" ]
then
	if ! test -f $TOP_DIR/benchmarks/tpcw/bin/datagen || \
		! test -f $TOP_DIR/benchmarks/tpcw/bin/appCache || \
		! test -f $TOP_DIR/benchmarks/tpcw/bin/appServer
	then
		install_tpcw
	else
		echo "TPC-W is installed"
	fi
fi
install_configs
