#!/usr/bin/env bash

input="$1"
tmp_prefix="$RANDOM"

if test -f "config.d/query_rate.conf.sh"
then
	source "config.d/query_rate.conf.sh"
else
	Com_delete="YES"
	Com_delete_multi="YES"
	Com_insert="YES"
	Com_insert_select="YES"
	Com_replace="YES"
	Com_replace_select="YES"
	Com_select="YES"
	Qcache_hits="YES"
	Com_all="YES"
fi

if test -f "config.d/sysstat.sh"
then
	source "config.d/sysstat.sh"
else
	echo "Can't find config.d/sysstat.sh"
	exit -1
fi

if ! test -f "$input"
then
	echo "Usage: `basename $0`: mysqladmin_ext.out"
	exit -1
fi

# Create temporary files with values
for i in Com_delete Com_delete_multi Com_insert Com_insert_select Com_replace Com_replace_select Com_select Qcache_hits
do
	grep $i $input | awk '{ print $4 }' > /tmp/"$tmp_prefix"_$i
done

# Generate .dat file

exec 10<> /tmp/"$tmp_prefix"_Com_delete
exec 11<> /tmp/"$tmp_prefix"_Com_delete_multi
exec 12<> /tmp/"$tmp_prefix"_Com_insert
exec 13<> /tmp/"$tmp_prefix"_Com_insert_select
exec 14<> /tmp/"$tmp_prefix"_Com_replace
exec 15<> /tmp/"$tmp_prefix"_Com_replace_select
exec 16<> /tmp/"$tmp_prefix"_Com_select
exec 17<> /tmp/"$tmp_prefix"_Qcache_hits

ts=$mysqladmin_ext_interval
while true
do
	read -u 10 Com_delete_value || break
	read -u 11 Com_delete_multi_value || break
	read -u 12 Com_insert_value || break
	read -u 13 Com_insert_select_value || break
	read -u 14 Com_replace_value || break
	read -u 15 Com_replace_select_value || break
	read -u 16 Com_select_value || break
	read -u 17 Qcache_hits_value || break
	let Com_delete_value=$Com_delete_value/$mysqladmin_ext_interval
	let Com_delete_multi_value=$Com_delete_multi_value/$mysqladmin_ext_interval
	let Com_insert_value=$Com_insert_value/$mysqladmin_ext_interval
	let Com_insert_select_value=$Com_insert_select_value/$mysqladmin_ext_interval
	let Com_replace_value=$Com_replace_value/$mysqladmin_ext_interval
	let Com_replace_select_value=$Com_replace_select_value/$mysqladmin_ext_interval
	let Com_select_value=$Com_select_value/$mysqladmin_ext_interval
	let Qcache_hits_value=$Qcache_hits_value/$mysqladmin_ext_interval
	let Com_all_value=$Com_delete_value+$Com_delete_multi_value+$Com_insert_value+$Com_insert_select_value+$Com_replace_value+$Com_replace_select_value+$Com_select_value+$Qcache_hits_value
	echo "$ts	$Com_delete_value	$Com_delete_multi_value	$Com_insert_value	$Com_insert_select_value	$Com_replace_value	$Com_replace_select_value	$Com_select_value	$Qcache_hits_value	$Com_all_value" >> /tmp/$tmp_prefix.dat
	let ts=$ts+$mysqladmin_ext_interval
done

# generate .gplot file

echo "set terminal png
set output \"$tmp_prefix.png\"
set title \"Query rate\"
set ylabel \"Query rate, (1/sec)\"
set xlabel \"time, sec\"
set yrange [0:]
set key right top
set datafile missing \"NaN\"
set grid xtics
set grid ytics" > /tmp/$tmp_prefix.gplot

echo -n "plot " >> /tmp/$tmp_prefix.gplot

k=1
sep=""
for i in Com_delete Com_delete_multi Com_insert Com_insert_select Com_replace Com_replace_select Com_select Qcache_hits All_queries
do
	echo -n "$sep '/tmp/$tmp_prefix.dat' using 1:$(($k + 1)) title '$i' with lines" >> /tmp/$tmp_prefix.gplot
	let k=$k+1
	sep=","
done

gnuplot /tmp/$tmp_prefix.gplot
rm /tmp/${tmp_prefix}*
echo "$tmp_prefix.png"
