#!/usr/bin/env bash

if ! test -d "$1"
then
	echo "Usage: `basename $0` <directory>"
	echo "	directory - a directory with benchwork config files"
	echo "	a configuration file shall be in form config.sh.(dbt2|sysbech|tpcc|whatever-is-supported)"
	exit -1
fi

config_dir="$1"

function get_n_vars {
local inv=1
local var_name
local xt
while true
do
        var_name="var${inv}"
        eval "xt=\$${var_name}"
        if test -z "${xt}"
        then
                break
        fi
        let inv=${inv}+1
done
echo $((${inv} - 1))
}

echo "====== A template for a test report  ======"
echo "===== Conclusion ====="
echo "===== Results ====="
for bch in `ls benchmarks`
do
	if test -f "$config_dir/config.sh.$bch"
	then
		echo "==== $bch ===="
		source "$config_dir/config.sh.$bch"
		results_dir="${PWD}${results_directory}/${results_prefix}/"`cat "${PWD}${results_directory}/${results_prefix}/"run_number`
		#read -t 60 -p "Enter the directory with $bch results[$results_dir]"
		#if [ "$REPLY" != "" ]
		#then
		#	results_dir="$REPLY"
		#fi
		echo $results_dir
		ln -fs "$config_dir"/chart.conf.sh.$bch chart.conf.sh
		ENABLE_ERRORS=YES ./chart.sh "$results_dir" > /dev/null 2>&1
		mv *.png "$config_dir"
		mv *.dat "$config_dir"
		cd "$config_dir"
		for i in *.png
		do
			echo "{{$i|}}"
			cat "`basename $i .png`.dat" | sed 's/\t/|/g' | sed 's/^./|&/' | sed 's/.$/&|/'
		done
		cd -
	fi
done
echo "===== Configuration ====="
echo "==== Hardware ===="

echo "Server: `hostname`"

echo "=== CPU ==="
echo "<code>"
echo "$ cat /proc/cpuinfo"
echo "`cat /proc/cpuinfo`"
echo "</code>"

echo "=== RAM ==="
echo "<code>"
echo "$ cat /proc/meminfo"
echo "`cat /proc/meminfo`"
echo "</code>"

echo "=== HDD ==="
echo "<code>"
echo "$ df -h"
echo "`df -h`"
echo "</code>"

echo "==== Operating System ===="
echo "<code>"
echo "$ uname -a"
echo "`uname -a`"
echo "</code>"

echo "==== MySQL ===="

echo "=== MySQL details ==="
echo "MySQL version"
for bch in `ls benchmarks`
do
	if test -f "$config_dir/config.sh.$bch"
        then
		# take on of config.sh and list all my.cnf from it
		source "$config_dir/config.sh.$bch"
		n="`get_n_vars`"
		i=1
		varied=0
		while [ "$i" -le "$n" ]
		do
			var_name="var${i}"
			eval "xt=\$${var_name}"
			if [ "$xt" = "mysq_server" ]
			then
				oldIFS="$IFS"
				IFS=","
				var_range_name="var${i}_range"
				eval "range=\$${var_range_name}"
				for v in $range
				do
					echo "  * $v"
				done
				varied=1
				IFS="$oldIFS"
			fi
			let i=$i+1
		done
		# If you are here then my.cnf isn't varied
		if [ $varied -eq 0 ]
		then
			echo "  * $mysql_server"
		fi
		break # loop untill any benchmark config is found
	fi
done
echo "=== MySQL configuration ==="
for bch in `ls benchmarks`
do
	if test -f "$config_dir/config.sh.$bch"
        then
		# take on of config.sh and list all my.cnf from it
		source "$config_dir/config.sh.$bch"
		n="`get_n_vars`"
		i=1
		varied=0
		while [ "$i" -le "$n" ]
		do
			var_name="var${i}"
			eval "xt=\$${var_name}"
			if [ "$xt" = "mysql_defaults" ]
			then
				oldIFS="$IFS"
				IFS=","
				var_range_name="var${i}_range"
				eval "range=\$${var_range_name}"
				for v in $range
				do
					echo "  * $v"
					echo "<code>"
					cat "db/etc/$v"
					echo "</code>"
				done
				varied=1
				IFS="$oldIFS"
			fi
			let i=$i+1
		done
		# If you are here then my.cnf isn't varied
		if [ $varied -eq 0 ]
		then
			echo "  * my.cnf"
			echo "<code>"
			cat "db/etc/my.cnf"
			echo "</code>"
		fi
		break # loop untill any benchmark config is found
	fi
done


echo "==== Benchwork configuration ===="
for bch in `ls benchmarks`
do
	if test -f "$config_dir/config.sh.$bch"
	then
		echo "=== $bch ==="
		echo "<code>"
		echo "`cat "$config_dir/config.sh.$bch"`"
		echo "</code>"
	fi
done

