/*
 * odbc_interaction_shopping_cart.c
 *
 * This file is released under the terms of the Artistic License.  Please see
 * the file LICENSE, included in this package, for details.
 *
 * Copyright (C) 2002 Mark Wong & Jenny Zhang &
 *                    Open Source Development Lab, Inc.
 *
 */

#include <odbc_interaction_shopping_cart.h>

#define STMT_CreateSC  "CALL createSC(?,?)"

int execute_shopping_cart(struct db_context_t *odbcc,
	struct shopping_cart_t *data)
{
	SQLRETURN rc;
	int i, j;

/*
*/
   long long  c_id;
   long long  sc_id;
   c_id  = data->c_id;
   sc_id = data->sc_id;

   if (data->sc_id == 0) {
#ifdef DEBUG
printf("START create shopping cart rc=%d c_id=%lld sc_id=%lld\n",rc,c_id,sc_id);
#endif
      rc = SQLPrepare(odbcc->hstmt, STMT_CreateSC, SQL_NTS);
      if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
      {
         LOG_ODBC_ERROR(SQL_HANDLE_STMT, odbcc->hstmt);
            return W_ERROR;
      }
      j = 1;
      rc = SQLBindParameter(odbcc->hstmt,
         j++, SQL_PARAM_INPUT, SQL_C_ULONG, SQL_INTEGER, 0, 0,
         &c_id, 0, NULL);
      if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
      {
         LOG_ODBC_ERROR(SQL_HANDLE_STMT, odbcc->hstmt);
            return W_ERROR;
      }
      rc = SQLBindParameter(odbcc->hstmt,
         j++, SQL_PARAM_OUTPUT, SQL_C_ULONG, SQL_INTEGER, 0, 0,
         &sc_id, 0, NULL);
      if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
      {
         LOG_ODBC_ERROR(SQL_HANDLE_STMT, odbcc->hstmt);
            return W_ERROR;
      }

      rc = SQLExecute(odbcc->hstmt);
      if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
      {
         LOG_ODBC_ERROR(SQL_HANDLE_STMT, odbcc->hstmt);
         return W_ERROR;
      }
      if (sc_id == 0) {
#ifdef DEBUG
printf("ERROR create shopping cart rc=%d c_id=%lld sc_id=%lld\n",rc,c_id,sc_id);
#endif
         LOG_ODBC_ERROR(SQL_HANDLE_STMT, odbcc->hstmt);
         return W_ERROR;
      } else {
         data->sc_id = sc_id;
      }
#ifdef DEBUG
printf("END   create shopping cart rc=%d c_id=%lld sc_id=%lld\n",rc,c_id,sc_id);
#endif
   }

	/* Perpare statement for shoppingcart interaction. */
	rc = SQLPrepare(odbcc->hstmt, STMT_SC, SQL_NTS);
	if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
	{
		LOG_ODBC_ERROR(SQL_HANDLE_STMT, odbcc->hstmt);
			return W_ERROR;
	}

	j = 1;
	/* Bind variables for shopping_cart interaction. */
	rc = SQLBindParameter(odbcc->hstmt,
		j++, SQL_PARAM_INPUT, SQL_C_ULONG, SQL_INTEGER, 0, 0,
		&data->c_id, 0, NULL);
	if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
	{
		LOG_ODBC_ERROR(SQL_HANDLE_STMT, odbcc->hstmt);
			return W_ERROR;
	}
	rc = SQLBindParameter(odbcc->hstmt,
		j++, SQL_PARAM_INPUT_OUTPUT, SQL_C_ULONG, SQL_INTEGER, 0, 0,
		&data->sc_id, 0, NULL);
	if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
	{
		LOG_ODBC_ERROR(SQL_HANDLE_STMT, odbcc->hstmt);
			return W_ERROR;
	}
	rc = SQLBindParameter(odbcc->hstmt,
		j++, SQL_PARAM_INPUT_OUTPUT, SQL_C_SSHORT, SQL_SMALLINT, 0, 0,
		&data->sc_size, 0, NULL);
	if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
	{
		LOG_ODBC_ERROR(SQL_HANDLE_STMT, odbcc->hstmt);
			return W_ERROR;
	}
	rc = SQLBindParameter(odbcc->hstmt,
		j++, SQL_PARAM_INPUT, SQL_C_SSHORT, SQL_SMALLINT, 0, 0,
		&data->add_flag, 0, NULL);
	if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
	{
		LOG_ODBC_ERROR(SQL_HANDLE_STMT, odbcc->hstmt);
			return W_ERROR;
	}
	rc = SQLBindParameter(odbcc->hstmt,
		j++, SQL_PARAM_INPUT, SQL_C_ULONG, SQL_INTEGER, 0, 0,
		&data->i_id, 0, NULL);
	if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
	{
		LOG_ODBC_ERROR(SQL_HANDLE_STMT, odbcc->hstmt);
			return W_ERROR;
	}
	rc = SQLBindParameter(odbcc->hstmt,
		j++, SQL_PARAM_INPUT, SQL_C_ULONG, SQL_INTEGER, 0, 0,
		&data->pp_data.i_id, 0, NULL);
	if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
	{
		LOG_ODBC_ERROR(SQL_HANDLE_STMT, odbcc->hstmt);
			return W_ERROR;
	}
	/*bind i_id,qty pairs*/
	for (i=0; i<SHOPPING_CART_ITEMS_MAX; i++)
	{
		rc = SQLBindParameter(odbcc->hstmt, j++,
		SQL_PARAM_INPUT, SQL_C_ULONG, SQL_INTEGER, 0,0,
		&data->sc_refresh[i].scl_i_id, 0, NULL);
		if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
		{
			LOG_ODBC_ERROR(SQL_HANDLE_STMT, odbcc->hstmt);
				return W_ERROR;
		}
		rc = SQLBindParameter(odbcc->hstmt, j++,
		SQL_PARAM_INPUT, SQL_C_SSHORT, SQL_SMALLINT, 0,0,
		&data->sc_refresh[i].scl_qty, 0, NULL);
		if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
		{
			LOG_ODBC_ERROR(SQL_HANDLE_STMT, odbcc->hstmt);
				return W_ERROR;
		}
	}
	/* bind shopping cart line info*/
	for (i=0; i<SHOPPING_CART_ITEMS_MAX; i++)
	{
		rc = SQLBindParameter(odbcc->hstmt,
			j++, SQL_PARAM_OUTPUT, SQL_C_ULONG, SQL_INTEGER, 0, 0, 
			&data->scl_data[i].scl_i_id, 0, NULL);
		if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
		{
			LOG_ODBC_ERROR(SQL_HANDLE_STMT, odbcc->hstmt);
				return W_ERROR;
		}
		rc = SQLBindParameter(odbcc->hstmt,
			j++, SQL_PARAM_OUTPUT, SQL_C_CHAR, SQL_VARCHAR, 0, 0,
			data->scl_data[i].i_title, sizeof(data->scl_data[i].i_title), NULL);
		if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
		{
			LOG_ODBC_ERROR(SQL_HANDLE_STMT, odbcc->hstmt);
				return W_ERROR;
		}
		rc = SQLBindParameter(odbcc->hstmt,
			j++, SQL_PARAM_OUTPUT, SQL_C_DOUBLE, SQL_DOUBLE, 0, 0,
			&data->scl_data[i].scl_cost, 0, NULL);
		if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
		{
			LOG_ODBC_ERROR(SQL_HANDLE_STMT, odbcc->hstmt);
				return W_ERROR;
		}
		rc = SQLBindParameter(odbcc->hstmt,
			j++, SQL_PARAM_OUTPUT, SQL_C_DOUBLE, SQL_DOUBLE, 0, 0,
			&data->scl_data[i].scl_srp, 0, NULL);
		if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
		{
			LOG_ODBC_ERROR(SQL_HANDLE_STMT, odbcc->hstmt);
				return W_ERROR;
		}
		rc = SQLBindParameter(odbcc->hstmt,
			j++, SQL_PARAM_OUTPUT, SQL_C_CHAR, SQL_VARCHAR, 0, 0,
			data->scl_data[i].i_backing,
			sizeof(data->scl_data[i].i_backing), NULL);
		if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
		{
			LOG_ODBC_ERROR(SQL_HANDLE_STMT, odbcc->hstmt);
				return W_ERROR;
		}
		rc = SQLBindParameter(odbcc->hstmt, j++,
			SQL_PARAM_OUTPUT, SQL_C_SSHORT, SQL_SMALLINT, 0,0,
			&data->scl_data[i].scl_qty, 0, NULL);
		if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
		{
			LOG_ODBC_ERROR(SQL_HANDLE_STMT, odbcc->hstmt);
				return W_ERROR;
		}
	}
	/* bind promo proc data*/
	for (i=0; i< PROMOTIONAL_ITEMS_MAX; i++)
	{
		rc = SQLBindParameter(odbcc->hstmt,
			j++, SQL_PARAM_OUTPUT, SQL_C_ULONG, SQL_INTEGER, 0, 0,
			&data->pp_data.i_related[i], 0, NULL);
		if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
		{
			LOG_ODBC_ERROR(SQL_HANDLE_STMT, odbcc->hstmt);
				return W_ERROR;
		}
		rc = SQLBindParameter(odbcc->hstmt,
			j++, SQL_PARAM_OUTPUT, SQL_C_ULONG, SQL_INTEGER, 0, 0,
			&data->pp_data.i_thumbnail[i], 0, NULL);
		if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
		{
			LOG_ODBC_ERROR(SQL_HANDLE_STMT, odbcc->hstmt);
				return W_ERROR;
		}
	}

	/* generate random item if add_flag == FALSE and empty (i_id, qty) pair */
	if (data->sc_size == 0 && data->add_flag == FALSE)
	{
		data->i_id = (UDWORD) get_random_int(item_count) + 1;
	}
	/* Generate random number for Promotional Processing. */
	data->pp_data.i_id = get_random_int(item_count) + 1;

	/* Execute stored procedure. */
	rc = SQLExecute(odbcc->hstmt);
#ifdef DEBUG
unsigned char *dumpptr;
dumpptr=(char *)data;
printf("shopping cart rc=%d c_id=%lld sc_id=%lld sc_size=%hd flag=%d\n",rc,data->c_id,data->sc_id,data->sc_size,data->add_flag);
/*
printf("%02x",*dumpptr++);
printf("%02x",*dumpptr++);
printf("%02x",*dumpptr++);
printf("%02x",*dumpptr++);
printf(" ");
printf("%02x",*dumpptr++);
printf("%02x",*dumpptr++);
printf("%02x",*dumpptr++);
printf("%02x",*dumpptr++);
printf("%02x",*dumpptr++);
printf("%02x",*dumpptr++);
printf("%02x",*dumpptr++);
printf("%02x",*dumpptr++);
printf(" ");
printf("%02x",*dumpptr++);
printf("%02x",*dumpptr++);
printf("%02x",*dumpptr++);
printf("%02x",*dumpptr++);
printf("%02x",*dumpptr++);
printf("%02x",*dumpptr++);
printf("%02x",*dumpptr++);
printf("%02x",*dumpptr++);
printf("\n");
dumpptr=(char *)&data->sc_id;
printf("%02x",*dumpptr++);
printf("%02x",*dumpptr++);
printf("%02x",*dumpptr++);
printf("%02x",*dumpptr++);
printf("%02x",*dumpptr++);
printf("%02x",*dumpptr++);
printf("%02x",*dumpptr++);
printf("%02x",*dumpptr++);
printf(" ");
printf("%02x",*dumpptr++);
printf("%02x",*dumpptr++);
printf("%02x",*dumpptr++);
printf("%02x",*dumpptr++);
printf("\n");
*/
#endif
	if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
	{
		LOG_ODBC_ERROR(SQL_HANDLE_STMT, odbcc->hstmt);
		return W_ERROR;
	}

	return OK;
}
