/*
 * interaction_admin_request.c
 *
 * This file is released under the terms of the Artistic License.  Please see
 * the file LICENSE, included in this package, for details.
 *
 * Copyright (C) 2002 Mark Wong & Jenny Zhang &
 *                    Open Source Development Lab, Inc.
 * Copyright (C) 2005 Hongwei Liu@SCS Sumisho Computer System corporation
 * Copyright (C) 2005 Masaki.Kuroda & Shuuichi Suzuki
 *                    UNIADEX, Ltd. 
 *
 * 28 february 2002
 * 06 July 2005
 */

#include <interaction_admin_request.h>

int execute_admin_request(struct db_context_t *odbcc,
	struct admin_request_t *data)
{
	SQLRETURN rc;
	int j = 0 ;
	char sql_cmd[512];

	//Create SQL Command
	memset( sql_cmd , 0x00 , sizeof( sql_cmd ) ) ;
	sprintf(sql_cmd,STMT_ADMIN_REQUEST, data->i_id);
	
	//Then, execute it.
	rc = SQLExecDirect(odbcc->hstmt,sql_cmd,SQL_NTS);
	if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO) {
		LOG_ODBC_ERROR(SQL_HANDLE_STMT, odbcc->hstmt);
		SQLCloseCursor(odbcc->hstmt);
		return W_ERROR;
	}
	
	/* Bind variables for Admin Request interaction. */
	j = 1;
	
	// i_srp
	rc = SQLBindCol(odbcc->hstmt,j++,SQL_C_DOUBLE,&data->i_srp, 0, NULL);
	if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
	{
		LOG_ODBC_ERROR(SQL_HANDLE_STMT, odbcc->hstmt);
		SQLCloseCursor(odbcc->hstmt);
		return W_ERROR;
	}
	
	// i_cost
	rc = SQLBindCol(odbcc->hstmt,j++,SQL_C_DOUBLE,&data->i_cost, 0, NULL);
	if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
	{
		LOG_ODBC_ERROR(SQL_HANDLE_STMT, odbcc->hstmt);
		SQLCloseCursor(odbcc->hstmt);
		return W_ERROR;
	}
	
	// i_title
	rc = SQLBindCol(odbcc->hstmt,j++,SQL_C_CHAR,data->i_title, sizeof(data->i_title), NULL);
	if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
	{
		LOG_ODBC_ERROR(SQL_HANDLE_STMT, odbcc->hstmt);
		SQLCloseCursor(odbcc->hstmt);
		return W_ERROR;
	}
	
	// i_image
#ifdef ORACLE_ODBC
	data->i_image = 0;
	rc = SQLBindCol(odbcc->hstmt,j++,SQL_C_ULONG,&data->i_image, 0, NULL);
#else
	rc = SQLBindCol(odbcc->hstmt,j++,SQL_C_UBIGINT,&data->i_image, 0, NULL);
#endif
	if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
	{
		LOG_ODBC_ERROR(SQL_HANDLE_STMT, odbcc->hstmt);
		SQLCloseCursor(odbcc->hstmt);
		return W_ERROR;
	}
	
	// i_thumbnail
#ifdef ORACLE_ODBC
	data->i_thumbnail = 0;
	rc = SQLBindCol(odbcc->hstmt,j++,SQL_C_ULONG,&data->i_thumbnail, 0, NULL);
#else
	rc = SQLBindCol(odbcc->hstmt,j++,SQL_C_UBIGINT,&data->i_thumbnail, 0, NULL);
#endif
	if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
	{
		LOG_ODBC_ERROR(SQL_HANDLE_STMT, odbcc->hstmt);
		SQLCloseCursor(odbcc->hstmt);
		return W_ERROR;
	}
	
	//a_fname
	rc = SQLBindCol(odbcc->hstmt,j++,SQL_C_CHAR,data->a_fname, sizeof(data->a_fname), NULL);
	if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
	{
		LOG_ODBC_ERROR(SQL_HANDLE_STMT, odbcc->hstmt);
		SQLCloseCursor(odbcc->hstmt);
		return W_ERROR;
	}
	
	//a_lname
	rc = SQLBindCol(odbcc->hstmt,j++,SQL_C_CHAR,data->a_lname, sizeof(data->a_lname), NULL);
	if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
	{
		LOG_ODBC_ERROR(SQL_HANDLE_STMT, odbcc->hstmt);
		SQLCloseCursor(odbcc->hstmt);
		return W_ERROR;
	}
	
	//get data
	rc = SQLFetch(odbcc->hstmt);
	if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO && rc!=SQL_NO_DATA_FOUND) {
		LOG_ODBC_ERROR(SQL_HANDLE_STMT, odbcc->hstmt);
		SQLCloseCursor(odbcc->hstmt);
		return W_ERROR;
	}
	
	SQLCloseCursor(odbcc->hstmt);
	
	return OK;
}
